@@ -181,9 +181,6 @@ class Host(object):
     Password for connecting to the host. (for sudo)
     """
 
-    # Terminal to report to use for interactive sessions
-    term = os.environ.get('TERM', 'xterm') # xterm, vt100, xterm-256color
-
     # Magic prompt. We expect this string to not appear in the stdout of
     # random programs. This makes it possible to automatically send the
     # correct password when sudo asks us.
@@ -293,6 +290,9 @@ class Host(object):
         else:
             result.append('cd %s && ' % cwd)
 
+        # Set TERM variable.
+        result.append("export TERM=%s && " % self.pty.get_term_var())
+
         # Prefix with variable assignments
         for var, value in self.host_context._env:
             #result.append('%s=%s ' % (var, value))
@@ -330,6 +330,7 @@ class Host(object):
         :param initial_input: When ``interactive``, send this input first to the host.
         """
         assert isinstance(command, basestring)
+        assert not initial_input or interactive # initial_input can only in case of interactive.
 
         logger = DummyLoggerInterface() if silent else self.logger
         pty = DummyPty() if silent else self.pty
@@ -340,7 +341,7 @@ class Host(object):
         # Run in PTY (Sudo usually needs to be run into a pty)
         if interactive:
             height, width = pty.get_size()
-            chan.get_pty(term=self.term, width=width, height=height)
+            chan.get_pty(term=self.pty.get_term_var(), width=width, height=height)
 
             # Keep size of local pty and remote pty in sync
             def set_size():
@@ -406,42 +407,13 @@ class Host(object):
 
             if interactive:
                 # Pty receive/send loop
-                result = self._posix_shell(chan, log_entry=log_entry, initial_input=initial_input)
+                result = self._posix_shell(chan, initial_input=initial_input)
             else:
                 # Read loop.
-                result = []
-                while True:
-                    # Before calling recv, call select to make sure
-                    # the channel is ready to be read. (Trick for
-                    # getting the SIGCHLD pipe of Localhost to work.)
-                    while True:
-                        r, w, e = select([chan], [], [], 5)
-                        if r:
-                            break
-                        else:
-                            print 'XXX: Select timed out, retrying...', r, w, e
-
-                    if chan in r:
-                        # Blocking call. Returns when data has been received or at
-                        # the end of the channel stream.
-                        try:
-                            data = chan.recv(1024)
-                        except IOError:
-                            # In case of localhost: application terminated,
-                            # caught in SIGCHLD, and closed slave PTY
-                            break
-
-                        if data:
-                            result += [data]
-                        else:
-                            break
-
-                result = ''.join(result)
-                log_entry.log_io(result)
+                result = self._read_non_interactive(chan)
 
                 #print result # I don't think we need to print the result of non-interactive runs
-                              # In any case self._run_silent_sudo should not
-                              # print anything.
+                              # In any case self._run_silent_sudo should not print anything.
 
             # Retrieve status code
             status_code = chan.recv_exit_status()
@@ -461,13 +433,17 @@ class Host(object):
     def _get_session(self):
         raise NotImplementedError
 
+    def _read_non_interactive(self, channel):
+        """ Read data from channel and return output. """
+        raise NotImplementedError
+
     def start_interactive_shell(self, command=None, initial_input=None):
         """
         Start an interactive bash shell.
         """
         raise NotImplementedError
 
-    def _posix_shell(self, chan, raw=True, log_entry=None, initial_input=None): # TODO: drop log_entry parameter: not used in logging backends.
+    def _posix_shell(self, chan, raw=True, initial_input=None):
         """
         Create a loop which redirects sys.stdin/stdout into this channel.
         The loop ends when channel.recv() returns 0.
@@ -514,9 +490,6 @@ class Host(object):
                         if len(x) == 0:
                             break
 
-                        # Log received characters
-                        log_entry.log_io(x)
-
                         # Write received characters to stdout and flush
                         while True:
                             try:
